
# Generate devel rpm
%global with_devel 0
# Build with debug info rpm
%global with_debug 0
# Run unit tests
%global with_tests 0
# Build test binaries
%global with_test_binaries 0

%define _unpackaged_files_terminate_build 0

%if 0%{?with_debug}
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif


%global import_path     istio.io/istio

%global istio_go_path   ISTIO
%global istio_go_src    %{istio_go_path}/src/%{import_path}
%global istio_release   %{istio_go_src}/out/linux_amd64

%if "%{dist}" != ".el7"
%global binaries        pilot-discovery pilot-agent istioctl
%else
%global binaries        istioctl
%endif

# Istio directories
%global istio_file_dir        %{_sysconfdir}/istio
%global istio_file_build_dir  %{buildroot}/%{istio_file_dir}

%global istio_artifact_file_dir   %{istio_file_dir}/artifacts
%global istio_artifact_build_dir  %{buildroot}/%{istio_artifact_file_dir}
# Istio-proxy directories
%global istio_pilot_artifact_file_dir  %{_sysconfdir}/pilot-agent/artifacts
%global istio_pilot_artifact_build_dir %{buildroot}/%{istio_pilot_artifact_file_dir}

# Use /usr/local as base dir, once upstream heavily depends on that
%global _prefix /usr/local
%global istio_main_version 1.8.5

Name:           istio
Version:        %istio_main_version
Release:        1%{?dist}
Summary:        An open platform to connect, manage, and secure microservices
License:        ASL 2.0
Vendor:         Oracle America
URL:            https://github.com/istio/istio

Source0:        %{name}-%{version}.tar.bz2
Source1:	istiorc
Source2:        buildinfo
Patch0:		run.sh.patch
Patch1:		Makefile.core.mk.patch
Patch2:		upgrade.go.patch
Patch3:		setup_env.sh.patch
Patch4:		egress-values.patch
Patch5:		ingress-values.patch
Patch6:         init.sh.patch

# e.g. el6 has ppc64 arch without gcc-go, so EA tag is required
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:%{ix86} x86_64 aarch64 %{arm}}
# If go_compiler is not set to 1, there is no virtual provide. Use golang instead.
BuildRequires:  golang >= 1.13.5
BuildRequires:  python2
BuildRequires:  hostname
BuildRequires:  helm

%if "%{dist}" != ".el7"
BuildRequires:  istio-proxy = %{version}
%endif

%description
Istio is an open platform that provides a uniform way to connect, manage
and secure microservices. Istio supports managing traffic flows between
microservices, enforcing access policies, and aggregating telemetry data,
all without requiring changes to the microservice code.

########### pilot-discovery ###############
%package pilot-discovery
Summary:  The istio pilot discovery
Requires: istio = %{version}-%{release}

%description pilot-discovery
Istio is an open platform that provides a uniform way to connect, manage
and secure microservices. Istio supports managing traffic flows between
microservices, enforcing access policies, and aggregating telemetry data,
all without requiring changes to the microservice code.

This package contains the pilot-discovery program.

pilot-discovery is the main pilot component and belongs to Control Plane.

########### pilot-agent ###############
%package pilot-agent
Summary:  The istio pilot agent
Requires: istio = %{version}-%{release}

%description pilot-agent
Istio is an open platform that provides a uniform way to connect, manage
and secure microservices. Istio supports managing traffic flows between
microservices, enforcing access policies, and aggregating telemetry data,
all without requiring changes to the microservice code.

This package contains the pilot-agent program.

pilot-agent is agent that talks to Istio pilot. It belongs to Data Plane.
Along with Envoy, makes up the proxy that goes in the sidecar along with applications.

########### istioctl ###############
%package istioctl
Summary:  The istio command line tool
Requires: istio = %{version}-%{release}

%description istioctl
Istio is an open platform that provides a uniform way to connect, manage
and secure microservices. Istio supports managing traffic flows between
microservices, enforcing access policies, and aggregating telemetry data,
all without requiring changes to the microservice code.

This package contains the istioctl program.

istioctl is the configuration command line utility.

%if 0%{?with_test_binaries}

########### tests ###############
%package pilot-tests
Summary:  Istio Pilot Test Binaries
Requires: istio = %{version}-%{release}

%description pilot-tests
Istio is an open platform that provides a uniform way to connect, manage
and secure microservices. Istio supports managing traffic flows between
microservices, enforcing access policies, and aggregating telemetry data,
all without requiring changes to the microservice code.

This package contains the binaries needed for pilot tests.

%endif

%if 0%{?with_devel}
%package devel
Summary:       %{summary}
BuildArch:     noarch

%description devel
Istio is an open platform that provides a uniform way to connect, manage
and secure microservices. Istio supports managing traffic flows between
microservices, enforcing access policies, and aggregating telemetry data,
all without requiring changes to the microservice code.

This package contains library source intended for
building other packages which use import path with
%{import_path} prefix.
%endif

%prep
mkdir -p %{istio_go_src}
tar xvf %{SOURCE0} -C %{istio_go_src} --strip=1
#proxy setup
mkdir -p %{istio_go_src}/ENVOY_BIN
# Move envoy to where istio expects it to be.  Stub this
# out for OL7 builds as envoy is only built for OL8
%if "%{dist}" != ".el7"
cp /usr/local/bin/envoy %{istio_go_src}/ENVOY_BIN/
cp %{SOURCE2} %{istio_go_src}/buildinfo
%else
touch %{istio_go_src}/ENVOY_BIN/envoy
%endif

# Apply run script patch to ensure that
# no invalid podman arguments are used
pushd %{istio_go_src}
%patch0
%patch1
%patch2
%if "%{dist}" == ".el7"
%patch3
%endif
%patch4
%patch5
%patch6
popd

which python || ln -s /usr/bin/python2 /usr/bin/python

%build
pushd %{istio_go_path}
export GOPATH=$(pwd)
popd

pushd %{istio_go_src}

# NOTE: Since our build pipline uses 'git archive' to tar the repo for the rpm build process
# we have to fake out the git history so istio will not mark our project as dirty in the version cmd

# Remove git archive files
rm -rf HEAD
rm -rf FETCH_HEAD
# Commit all local changes to keep the project clean
git config --global user.email "o@oracle.com"
git config --global user.name "oracle"
git init
git add .
git commit -a -m "Oracle Path Files"
git tag %{istio_main_version} -m 'Oracle Build'

# Istio vars to generate yaml
export REGISTRY="container-registry.oracle.com"
# This release is a tech preview so the default generated deploy yaml has to point to the developer namespace
export REGISTRY_NAMESPACE="olcne-developer"
export OUTPUT_BASE_DIR="oracle"
# Shared Arg between Istio build and generating istio deploy yaml
export ISTIO_VERSION=%{istio_main_version}
export VERSION=%{istio_main_version}
# Override Istio build Args to populate istioctl version cmd
export ISTIO_DOCKER_HUB=${REGISTRY}/${REGISTRY_NAMESPACE}
# To consume Oracle built envoy binary instead of fetching from upstream
# For containerized builds, /work in the container corresponds to
# %{istio_go_src} on the host
export ISTIO_ENVOY_LOCAL=/work/ENVOY_BIN/envoy
export ISTIO_ENVOY_LOCAL_PATH=/work/ENVOY_BIN/envoy
# Use podman instead of Docker
export CONTAINER_CLI=podman
export DOCKER_SOCKET_MOUNT=" "
# Jenkins pipeline has DEBUG=false which clobbers the Makefile
make USE_LOCAL_PROXY=1 DEBUG=0 gen-charts
make USE_LOCAL_PROXY=1 DEBUG=0 %{binaries} && echo "OK"

%if 0%{?with_test_binaries}
make DEBUG=0 test-bins
%endif

# Build istio deploy yaml
#NOTE: Commenting it as we are using olcne installer to deploy istio as a module
#chmod +x generate_istio_deploy_yaml.sh
#./generate_istio_deploy_yaml.sh

popd

%install
mkdir -p $RPM_BUILD_ROOT%{_bindir}
shopt -s dotglob
pushd %{istio_release}
bins=(%{binaries})
%if 0%{?with_debug}
  for i in "${bins[@]}"; do
    cp -pav $i $RPM_BUILD_ROOT%{_bindir}/
  done
%else
  mkdir stripped
  for i in "${bins[@]}"; do
    echo stripping: $i
    strip -o stripped/$i -s $i
    cp -pav stripped/$i $RPM_BUILD_ROOT%{_bindir}/
  done
%endif

# Files used with pilot
%if "%{dist}" != ".el7"
install -d -p %{istio_pilot_artifact_build_dir}
install -d -p %{istio_artifact_build_dir}

install %{_builddir}/%{istio_go_src}/tools/packaging/common/envoy_bootstrap.json     %{istio_pilot_artifact_build_dir}
install %{_builddir}/%{istio_go_src}/tools/packaging/common/gcp_envoy_bootstrap.json %{istio_pilot_artifact_build_dir}
# Files used to deploy istio
#install %{_builddir}/%{istio_go_src}/generate_istio_deploy_yaml.sh                  %{istio_file_build_dir}
#install %{_builddir}/%{istio_go_src}/oracle/istio.yaml                              %{istio_artifact_build_dir}
#install %{_builddir}/%{istio_go_src}/oracle/istio-crds.yaml                         %{istio_artifact_build_dir}
%endif

%if 0%{?with_test_binaries}
cp -pav %{istio_release}/{pilot-test-server,pilot-test-client,pilot-test-eurekamirror} $RPM_BUILD_ROOT%{_bindir}/
%endif

%if 0%{?with_tests}

%check
cd %{istio_go_path}
export GOPATH=$(pwd):%{gopath}
export GOPATH=$(pwd):$GOPATH
pushd %{istio_go_src}
make DEBUG=0 localTestEnv test
make DEBUG=0 localTestEnvCleanup
popd

%endif

# source codes for building projects
%if 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
echo "%%dir %%{gopath}/src/%%{import_path}/." >> devel.file-list
# find all *.go but no *_test.go files and generate devel.file-list
for file in $(find . \( -iname "*.go" -or -iname "*.s" \) \! -iname "*_test.go") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

%if 0%{?with_devel}
sort -u -o devel.file-list devel.file-list
%endif

#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files
%license %{istio_go_src}/LICENSE %{istio_go_src}/THIRD_PARTY_LICENSES.txt
%doc     %{istio_go_src}/README.md
# Files used to deploy istio
#%{istio_artifact_file_dir}/istio-crds.yaml
#%{istio_artifact_file_dir}/istio.yaml
#%{istio_file_dir}/generate_istio_deploy_yaml.sh

%if "%{dist}" != ".el7"
%files pilot-discovery
%{_bindir}/pilot-discovery
%license %{istio_go_src}/LICENSE %{istio_go_src}/THIRD_PARTY_LICENSES.txt

%files pilot-agent
%{_bindir}/pilot-agent
# Files used with pilot
%{istio_pilot_artifact_file_dir}/envoy_bootstrap.json
%{istio_pilot_artifact_file_dir}/gcp_envoy_bootstrap.json
%license %{istio_go_src}/LICENSE %{istio_go_src}/THIRD_PARTY_LICENSES.txt
%endif

%files istioctl
%{_bindir}/istioctl
%license %{istio_go_src}/LICENSE %{istio_go_src}/THIRD_PARTY_LICENSES.txt

%if 0%{?with_test_binaries}
%files pilot-tests
%{_bindir}/pilot-test-server
%{_bindir}/pilot-test-client
%{_bindir}/pilot-test-eurekamirror
%endif

%if 0%{?with_devel}
%files devel -f devel.file-list
%license %{istio_go_src}/LICENSE %{istio_go_src}/THIRD_PARTY_LICENSES.txt
%doc %{istio_go_src}/README.md %{istio_go_src}/DEV-*.md %{istio_go_src}/CONTRIBUTING.md
%dir %{gopath}/src/%{provider}.%{provider_tld}/%{project}
%endif

%changelog
* Fri Apr 23 2021 michthom <michael.a.thompson@oracle.com> - 1.8.5-1
- Addresses CVE-2021-28683, CVE-2021-28682 and CVE-2021-29258
